#!/usr/bin/env python

import os
import re
import string
import time
import socket
import codecs
import cgi

class FSSError(Exception):
	def __init__(self, msg):
		self.__msg = "FSS: " + msg
	def __str__(self):
		return self.__msg
	msg = __str__

class FSS:
	def __init__(self,socket="/tmp/fastsmb.sock"):
		""" Set variables and cook socket """
		self.path = socket
		self.socket = None
		self.connect()

	def __del__(self):
		""" Destructor, close socket """
		self.disconnect()

	def connect(self):
		sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
		sock.connect(self.path)
		if (self.socket != None):
			self.socket.close()
		sock.settimeout(3);
		self.socket = codecs.EncodedFile(sock.makefile(), "utf8", "KOI8-R")
		if (self.socket == None):
			raise FSSError("Unable to connect to server")

	def disconnect(self):
		if (self.socket == None):
			return
		self.socket.close()
		self.socket = None

	def request(self, req):
		self.socket.write(req + "\n")
		self.socket.flush()

	def get_line(self):
		return string.rstrip(self.socket.readline(), "\n")
	__str__ = get_line

class FSSXML:
	def __init__(self):
		self.body = "<?xml version='1.0' encoding='utf8'?>\n"
		self.stack = []
	def push(self, tag, attr=""):
		self.body += "<"+tag+" "+attr+">"
		self.stack.append(tag)
	def pop(self, tag=""):
		while len(self.stack):
			tmp = self.stack.pop()
			self.body += "</"+tmp+">\n"
			if (tag == "" or tag == tmp):
				break
	def data(self, data):
		self.body += data;
	def xml(self):
		if len(self.stack):
			self.pop(self.stack[0])
		return self.body
	__str__ = xml

class FSSReq:
	def __init__(self):
		self.sum = 0
		self.offset = 0
	def init_common(self, array):
		self.cgi = array
		self.restype = self.get("restype", default = "all")
		self.count = self.get_int("count", default = 100)
		self.start = self.get_int("start", default = 0)
		self.end = self.start + 100;

	def init_search(self, array):
		self.init_common(array)
		self.what = self.get("what", fail = True, escape = False)
	def init_showdir(self, array):
		self.init_common(array)
		self.host = self.get_int("host", fail = True)
		self.where = self.get_int("where", fail = True)

	def get_int(self, what, array = None, default = 0, fail = False):
		tmp = self.get(what = what, array = array, fail = fail)
		if len(tmp):
			return int(tmp)
		return default

	def get(self, what, array = None, default = "", fail = False, escape = True):
		if not array:
			array = self.cgi
		try:
			if escape:
				return cgi.escape(array[what][0], True)
			else:
				return array[what][0]
		except Exception,e:
			if fail:
				raise e
			else:
				return default
	def url(self):
		return "what=%s&amp;go=%s&amp;restype=%s&amp;" % (cgi.escape(self.what, True), self.get("go"), self.get("restype", default="all"))

	def q_query_id(self, line):
		m = re.match("^QUERY_ID ([0-9]+) HINT (.+)", line)
		if (m == None):
			raise FSSError("Invalid QUERY_ID response: %s")
		self.query_id = int(m.group(1))
		self.hint = m.group(2)
	def q_count(self, line):
		m = re.match("^COUNT ([0-9]+) ([0-9]+) ([0-9]+(\.[0-9]*)?)", line)
		if (m == None):
			raise FSSError("Invalid COUNT response: %s" % line)

		self.matched = int(m.group(1))
		self.returned = int(m.group(2))
		self.search_time = m.group(3)


	def q_file_short(self, line, xml):
		(id, parent_id, name, size, is_dir, res_type) = string.split(line, "\t");

		if id == self.root_id:
			return

		xml.push("fss:file", "mode='short' id='%s' host='%s' parent='%s'" % (id, self.host, parent_id))

		self.sum += int(size)
		self.offset += 1

		if (is_dir != "0"):
			xml.data("<fss:filemeta name='dir'/>\n");
		xml.data("<fss:filemeta name='offset'>%s</fss:filemeta>\n" % self.offset);
		xml.data("<fss:filemeta name='size'>%s</fss:filemeta>\n" % size);
		xml.data("<fss:filemeta name='filename'>%s</fss:filemeta>\n" % cgi.escape(name));
		xml.pop("fss:file");
	def q_file(self, line, xml):
		(id, parent_id, name, size, is_dir, res_type,
				path, workgroup, host_id, proto, host_name, host_ip, status) = string.split(line, "\t")

		self.sum += int(size)
		self.offset += 1

		self.path = path
		self.file_name = name

		self.size = size

		xml.push("fss:file", "id='%s' host='%s' parent='%s'" % (id, host_id, parent_id))
		if (is_dir != "0"):
			xml.data("<fss:filemeta name='dir'/>\n");
		xml.data("<fss:filemeta name='offset'>%s</fss:filemeta>\n" % self.offset);
		xml.data("<fss:filemeta name='workgroup'>%s</fss:filemeta>\n" % workgroup);
		xml.data("<fss:filemeta name='size'>%s</fss:filemeta>\n" % size);
		xml.data("<fss:filemeta name='status'>%s</fss:filemeta>\n" % status);
		xml.data("<fss:filemeta name='proto'>%s</fss:filemeta>\n" % proto);
		xml.data("<fss:filemeta name='hostname'>%s</fss:filemeta>\n" % host_name);
		xml.data("<fss:filemeta name='ip'>%s</fss:filemeta>\n" % host_ip);
		xml.data("<fss:filemeta name='filename'>%s</fss:filemeta>\n" % cgi.escape(name));
		xml.data("<fss:filemeta name='path'>%s</fss:filemeta>\n" % cgi.escape(path));
		xml.pop("fss:file");
	def q_host(self, line):
		(self.host_id, self.proto, self.host_name, self.host_ip, self.status, self.workgroup, self.root_id, self.max_id) = string.split(line, "\t")

